@@ -35,8 +35,8 @@ type TP = GenParser Char ()
 
 texMathDef :: LanguageDef st
 texMathDef = LanguageDef
-   { commentStart   = ""
-   , commentEnd     = ""
+   { commentStart   = "\\label{"
+   , commentEnd     = "}"
    , commentLine    = "%"
    , nestedComments = False
    , identStart     = letter
@@ -77,6 +77,7 @@ formula :: TP [Exp]
 formula = do
   whiteSpace
   f <- many expr
+  whiteSpace
   eof
   return f
 
@@ -97,7 +98,7 @@ inbraces :: TP Exp
 inbraces = liftM EGrouped (braces $ many $ notFollowedBy (char '}') >> expr)
 
 texToken :: TP Exp
-texToken = inbraces <|> inbrackets <|>
+texToken = texSymbol <|> inbraces <|> inbrackets <|>
              do c <- anyChar
                 spaces
                 return $ if isDigit c
@@ -322,22 +323,34 @@ command = try $ char '\\' >> liftM ('\\':) (identifier <|> lexeme (count 1 anyCh
 unaryOps :: [String]
 unaryOps = ["\\sqrt", "\\surd"]
 
+-- Note: cal and scr are treated the same way, as unicode is lacking such two different sets for those.
 textOps :: M.Map String (String -> Exp)
 textOps = M.fromList
-          [ ("\\textrm", EText TextNormal . parseText)
-          , ("\\mathrm", EText TextNormal)
-          , ("\\text",   EText TextNormal . parseText)
-          , ("\\mbox",   EText TextNormal)
-          , ("\\mathbf", EText TextBold)
-          , ("\\textbf", EText TextBold . parseText)
-          , ("\\mathit", EText TextItalic)
-          , ("\\textit", EText TextItalic . parseText)
-          , ("\\mathtt", EText TextMonospace)
-          , ("\\texttt", EText TextMonospace)
-          , ("\\mathsf", EText TextSansSerif)
-          , ("\\mathbb", \e -> maybe (EText TextDoubleStruck e) (ESymbol Pun) (M.lookup e mathbb))
-          , ("\\mathcal", \e -> maybe (EText TextScript e) (ESymbol Pun) (M.lookup e mathcal))
-          , ("\\mathfrak", EText TextFraktur)
+          [ ("\\textrm",     EText TextNormal . parseText)
+          , ("\\mathrm",     EText TextNormal)
+          , ("\\mathup",     EText TextNormal)
+          , ("\\text",       EText TextNormal . parseText)
+          , ("\\mbox",       EText TextNormal)
+          , ("\\mathbf",     EText TextBold)
+          , ("\\mathbfup",   EText TextBold)
+          , ("\\textbf",     EText TextBold . parseText)
+          , ("\\mathit",     EText TextItalic)
+          , ("\\textit",     EText TextItalic . parseText)
+          , ("\\mathtt",     EText TextMonospace)
+          , ("\\texttt",     EText TextMonospace)
+          , ("\\mathsf",     EText TextSansSerif)
+          , ("\\mathsfup",   EText TextSansSerif)
+          , ("\\mathbb",     EText TextDoubleStruck)
+          , ("\\mathcal",    EText TextScript)
+          , ("\\mathscr",    EText TextScript)
+          , ("\\mathfrak",   EText TextFraktur)
+          , ("\\mathbfit",   EText TextBoldItalic)
+          , ("\\mathbfsfup", EText TextBoldSansSerif)
+          , ("\\mathbfsfit", EText TextBoldSansSerifItalic)
+          , ("\\mathbfscr",  EText TextBoldScript)
+          , ("\\mathbffrak", EText TextBoldFraktur)
+          , ("\\mathbfcal",  EText TextBoldScript)
+          , ("\\mathsfit",   EText TextSansSerifItalic)
           ]
 
 parseText :: String -> String
@@ -364,8 +377,8 @@ diacriticals = M.fromList
                , ("\\grave", \e -> EOver e (ESymbol Accent "\x0060"))
                , ("\\breve", \e -> EOver e (ESymbol Accent "\x02D8"))
                , ("\\check", \e -> EOver e (ESymbol Accent "\x02C7"))
-               , ("\\dot", \e -> EOver e (ESymbol Accent "."))
-               , ("\\ddot", \e -> EOver e (ESymbol Accent ".."))
+               , ("\\dot", \e -> EOver e (ESymbol Accent "\x307"))
+               , ("\\ddot", \e -> EOver e (ESymbol Accent "\x308"))
                , ("\\mathring", \e -> EOver e (ESymbol Accent "\x00B0"))
                , ("\\vec", \e -> EOver e (ESymbol Accent "\x20D7"))
                , ("\\overrightarrow", \e -> EOver e (ESymbol Accent "\x20D7"))
@@ -556,24 +569,10 @@ symbols = M.fromList [
            , ("\\vartheta", EIdentifier "\x03D1")
            , ("\\Theta", ESymbol Op "\x0398")
            , ("\\upsilon", EIdentifier "\x03C5")
+           , ("\\Upsilon", EIdentifier "\x03A5")
            , ("\\xi", EIdentifier "\x03BE")
            , ("\\Xi", ESymbol Op "\x039E")
            , ("\\zeta", EIdentifier "\x03B6")
-           , ("\\frac12", ESymbol Ord "\x00BD")
-           , ("\\frac14", ESymbol Ord "\x00BC")
-           , ("\\frac34", ESymbol Ord "\x00BE")
-           , ("\\frac13", ESymbol Ord "\x2153")
-           , ("\\frac23", ESymbol Ord "\x2154")
-           , ("\\frac15", ESymbol Ord "\x2155")
-           , ("\\frac25", ESymbol Ord "\x2156")
-           , ("\\frac35", ESymbol Ord "\x2157")
-           , ("\\frac45", ESymbol Ord "\x2158")
-           , ("\\frac16", ESymbol Ord "\x2159")
-           , ("\\frac56", ESymbol Ord "\x215A")
-           , ("\\frac18", ESymbol Ord "\x215B")
-           , ("\\frac38", ESymbol Ord "\x215C")
-           , ("\\frac58", ESymbol Ord "\x215D")
-           , ("\\frac78", ESymbol Ord "\x215E")
            , ("\\pm", ESymbol Bin "\x00B1")
            , ("\\mp", ESymbol Bin "\x2213")
            , ("\\triangleleft", ESymbol Bin "\x22B2")
@@ -768,130 +767,3 @@ symbols = M.fromList [
            , ("\\tanh", EMathOperator "tanh")
            ]
 
--- MathML has a mathvariant attribute which is unimplemented in Firefox
---    (see https://bugzilla.mozilla.org/show_bug.cgi?id=114365)
--- Therefore, we translate mathcal to unicode symbols directly.
--- This list is from http://www.w3.org/TR/MathML2/script.html
-mathcal :: M.Map String String
-mathcal = M.fromList [
-             ("A", "\x1D49C")
-           , ("B", "\x0212C")
-           , ("C", "\x1D49E")
-           , ("D", "\x1D49F")
-           , ("E", "\x02130")
-           , ("F", "\x02131")
-           , ("G", "\x1D4A2")
-           , ("H", "\x0210B")
-           , ("I", "\x02110")
-           , ("J", "\x1D4A5")
-           , ("K", "\x1D4A6")
-           , ("L", "\x02112")
-           , ("M", "\x02133")
-           , ("N", "\x1D4A9")
-           , ("O", "\x1D4AA")
-           , ("P", "\x1D4AB")
-           , ("Q", "\x1D4AC")
-           , ("R", "\x0211B")
-           , ("S", "\x1D4AE")
-           , ("T", "\x1D4AF")
-           , ("U", "\x1D4B0")
-           , ("V", "\x1D4B1")
-           , ("W", "\x1D4B2")
-           , ("X", "\x1D4B3")
-           , ("Y", "\x1D4B4")
-           , ("Z", "\x1D4B5")
-           , ("a", "\x1D4B6")
-           , ("b", "\x1D4B7")
-           , ("c", "\x1D4B8")
-           , ("d", "\x1D4B9")
-           , ("e", "\x0212F")
-           , ("f", "\x1D4BB")
-           , ("g", "\x0210A")
-           , ("h", "\x1D4BD")
-           , ("i", "\x1D4BE")
-           , ("j", "\x1D4BF")
-           , ("k", "\x1D4C0")
-           , ("l", "\x1D4C1")
-           , ("m", "\x1D4C2")
-           , ("n", "\x1D4C3")
-           , ("o", "\x02134")
-           , ("p", "\x1D4C5")
-           , ("q", "\x1D4C6")
-           , ("r", "\x1D4C7")
-           , ("s", "\x1D4C8")
-           , ("t", "\x1D4C9")
-           , ("u", "\x1D4CA")
-           , ("v", "\x1D4CB")
-           , ("w", "\x1D4CC")
-           , ("x", "\x1D4CD")
-           , ("y", "\x1D4CE")
-           , ("z", "\x1D4CF")
-           ]
-
--- Similar to mathcal above, we translate manually.
--- This list is from http://www.w3.org/TR/MathML2/double-struck.html
-mathbb :: M.Map String String
-mathbb = M.fromList [
-             ("A", "\x1D538")
-           , ("B", "\x1D539")
-           , ("C", "\x02102")
-           , ("D", "\x1D53B")
-           , ("E", "\x1D53C")
-           , ("F", "\x1D53D")
-           , ("G", "\x1D53E")
-           , ("H", "\x0210D")
-           , ("I", "\x1D540")
-           , ("J", "\x1D541")
-           , ("K", "\x1D542")
-           , ("L", "\x1D543")
-           , ("M", "\x1D544")
-           , ("N", "\x02115")
-           , ("O", "\x1D546")
-           , ("P", "\x02119")
-           , ("Q", "\x0211A")
-           , ("R", "\x0211D")
-           , ("S", "\x1D54A")
-           , ("T", "\x1D54B")
-           , ("U", "\x1D54C")
-           , ("V", "\x1D54D")
-           , ("W", "\x1D54E")
-           , ("X", "\x1D54F")
-           , ("Y", "\x1D550")
-           , ("Z", "\x02124")
-           , ("a", "\x1D552")
-           , ("b", "\x1D553")
-           , ("c", "\x1D554")
-           , ("d", "\x1D555")
-           , ("e", "\x1D556")
-           , ("f", "\x1D557")
-           , ("g", "\x1D558")
-           , ("h", "\x1D559")
-           , ("i", "\x1D55A")
-           , ("j", "\x1D55B")
-           , ("k", "\x1D55C")
-           , ("l", "\x1D55D")
-           , ("m", "\x1D55E")
-           , ("n", "\x1D55F")
-           , ("o", "\x1D560")
-           , ("p", "\x1D561")
-           , ("q", "\x1D562")
-           , ("r", "\x1D563")
-           , ("s", "\x1D564")
-           , ("t", "\x1D565")
-           , ("u", "\x1D566")
-           , ("v", "\x1D567")
-           , ("w", "\x1D568")
-           , ("x", "\x1D569")
-           , ("y", "\x1D56A")
-           , ("z", "\x1D56B")
-           , ("0", "\x1D7D8")
-           , ("1", "\x1D7D9")
-           , ("2", "\x1D7DA")
-           , ("3", "\x1D7DB")
-           , ("4", "\x1D7DC")
-           , ("5", "\x1D7DD")
-           , ("6", "\x1D7DE")
-           , ("7", "\x1D7DF")
-           , ("8", "\x1D7E0")
-           , ("9", "\x1D7E1")
-           ]
